@@ -137,7 +137,7 @@ time_series.mds_da <- function(
 
   # Filter device-events and exposure to the relevant levels
   # --------------------------------------------------------
-  # Device
+  # Device - Primary
   if (analysis$device_level == "All"){
     devlvl <- unique(deviceevents[[names(analysis$device_level)]])
   } else devlvl <- analysis$device_level
@@ -148,7 +148,7 @@ time_series.mds_da <- function(
     thes <- thes[thes[[names(analysis$exp_device_level)]] %in%
                    analysis$exp_device_level, ]
   }
-  # Event
+  # Event - Primary
   if (analysis$event_level == "All"){
     evlvl <- unique(this[[names(analysis$event_level)]])
   } else evlvl <- analysis$event_level
@@ -168,59 +168,66 @@ time_series.mds_da <- function(
 
   # Identify the type of analysis
   # -----------------------------
+  # Note: In the future for covariate by device by event level analysis (3D),
+  # atype may be modified to return a vector of c("iscov", "isdev", "isev")
   if (analysis$covariate_level != "All"){
     # Covariate level analysis
     if (analysis$event_level != "All"){
       # Covariate by event level analysis
-      atype <- c("iscov", "isev")
+      atype <- stats::setNames(c("iscov", "isev"), c("Covariate", "Event"))
     } else if (analysis$device_level != "All"){
       # Covariate by device level analysis
-      atype <- c("iscov", "isdev")
+      atype <-  stats::setNames(c("iscov", "isdev"), c("Covariate", "Device"))
     } else{
       # Covariate only analysis
-      atype <- "iscov"
+      atype <-  stats::setNames("iscov", c("Covariate"))
     }
   } else{
     # Non-covariate level analysis
     if (analysis$event_level != "All"){
       # Device by event level analysis
-      atype <- c("isdev", "isev")
+      atype <-  stats::setNames(c("isdev", "isev"), c("Device", "Event"))
     } else{
       # Device only analysis
-      atype <- "isdev"
+      atype <- stats::setNames("isdev", c("Device"))
     }
   }
   dpa <- length(atype) > 1
 
-  # Filter device-events to 1-level up hierarchy, if needed
-  # -------------------------------------------------------
+  # Filter to 1-level up hierarchy, if needed
+  # -----------------------------------------
   nextdev <- nextev <- NULL
   if (dpa & use_hierarchy){
     if ("isdev" %in% atype){
-      # Filter to 1-level up device hierarchy
+      # Filter device-events to 1-level up device hierarchy
       nextdev <- next_dev(names(analysis$device_level))
-      if (nextdev %in% names(this)){
+      if (nextdev %in% names(this) & !is.na(analysis$device_1up) &
+        nextdev == names(analysis$device_1up)){
         nmiss <- sum(is.na(this[[nextdev]]))
         if (nmiss > 0){
           warning(paste("Dropping", nmiss, "records with missing", nextdev))
           this <- this[!is.na(this[[nextdev]]), ]
         }
-        nextdev <- stats::setNames(this[this$isdev == T, ][[nextdev]][1],
-                                   nextdev)
-        this <- this[this[[names(nextdev)]] == nextdev, ]
+        this <- this[this[[nextdev]] %in% analysis$device_1up, ]
+        # Also filter exposures
+        if (nrow(thes) > 0 & !is.na(analysis$exp_device_1up) &
+            analysis$exp_device_1up != "All" &
+            nextdev == names(analysis$exp_device_1up)){
+          thes <- thes[thes[[nextdev]] %in% analysis$exp_device_1up, ]
+        }
       }
     }
     if ("isev" %in% atype){
-      # Filter to 1-level up event hierarchy
+      # Filter device-events to 1-level up event hierarchy
       nextev <- next_ev(names(analysis$event_level))
-      if (nextev %in% names(this)){
+      if (nextev %in% names(this) & !is.na(analysis$event_1up) &
+          nextev == names(analysis$event_1up)){
         nmiss <- sum(is.na(this[[nextev]]))
         if (nmiss > 0){
           warning(paste("Dropping", nmiss, "records with missing", nextev))
           this <- this[!is.na(this[[nextev]]), ]
         }
-        nextev <- stats::setNames(this[this$isev == T, ][[nextev]][1], nextev)
-        this <- this[this[[names(nextev)]] == nextev, ]
+        this <- this[this[[nextev]] %in% analysis$event_1up]
       }
     }
   }
@@ -276,47 +283,74 @@ time_series.mds_da <- function(
 
   # Define output class attributes
   # ------------------------------
-  # Level of nA
-  nA <- lapply(atype, function(x){
-    if (x == "isdev"){
-      analysis$device_level
-    } else if (x == "isev"){
-      analysis$event_level
-    } else if (x == "iscov"){
-      stats::setNames(analysis$covariate_level, analysis$covariate)
-    }
-  })
-  # Level of the entire contingency table
-  if (dpa){
-    nABCD <- lapply(atype, function(x){
-      if (x == "isdev"){
-        if (is.null(nextdev)){
-          stats::setNames("All", names(analysis$device_level))
-        } else nextdev
-      } else if (x == "isev"){
-        if (is.null(nextev)){
-          stats::setNames("All", names(analysis$event_level))
-        } else nextev
-      } else if (x == "iscov"){
-        stats::setNames("All", analysis$covariate)
+  # Determine if 1-Ups exist semantically
+  dev_diff <- !is.na(analysis$device_1up) & names(analysis$device_level) !=
+    names(analysis$device_1up)
+  ev_diff <- !is.na(analysis$event_1up) & names(analysis$event_level) !=
+    names(analysis$event_1up)
+  # Construct the labels for the analysis levels and (if dpa) contingency table
+  for (i in c(1:length(atype))){
+    if (names(atype)[i] == "Covariate"){
+      nhere <- analysis$covariate
+      lab <- paste(nhere, analysis$covariate_level)
+      notlab <- paste(nhere, "NOT", analysis$covariate_level)
+    } else if (names(atype)[i] == "Device"){
+      nhere <- analysis$device_level_source
+      lab <- paste(nhere, analysis$device_level)
+      notlab <- paste(nhere, "NOT", analysis$device_level)
+      # 1 up device presence
+      if (dev_diff){
+        lab1up <- paste("WITHIN", analysis$device_1up_source,
+                        analysis$device_1up)
+        nhere <- paste(nhere, lab1up)
+        lab <- paste(lab, lab1up)
+        notlab <- paste(notlab, lab1up)
       }
-    })
-  } else nABCD <- NULL
-  # nA-nD English labels
-  nLabels <- list(nA=sapply(nA, function(x) paste0(names(x), ":", x)))
+    } else if (names(atype)[i] == "Event"){
+      nhere <- analysis$event_level_source
+      lab <- paste(nhere, analysis$event_level)
+      notlab <- paste(nhere, "NOT", analysis$event_level)
+      # 1 up event presence
+      if (ev_diff){
+        lab1up <- paste("WITHIN", analysis$event_1up_source,
+                        analysis$event_1up)
+        nhere <- paste(nhere, lab1up)
+        lab <- paste(lab, lab1up)
+        notlab <- paste(notlab, lab1up)
+      }
+    }
+    # Assign row and columns (only 2D analysis support so far)
+    if (i == 1){
+      nRow <- nhere
+      nA <- lab
+      nB <- lab
+      nC <- notlab
+      nD <- notlab
+      title <- nRow
+    } else if (i == 2){
+      nCol <- nhere
+      nA <- paste0(nA, ":", lab)
+      nB <- paste0(nB, ":", notlab)
+      nC <- paste0(nC, ":", lab)
+      nD <- paste0(nD, ":", notlab)
+      title <- paste(title, "by", nCol)
+    }
+  }
+  # Save DPA details
   if (dpa){
-    nLabels$rows <- paste0(names(nABCD[[1]]), ":", nABCD[[1]])
-    nLabels$cols <- paste0(names(nABCD[[2]]), ":", nABCD[[2]])
+    dpa_dtl <- list(nA=nA, nB=nB, nC=nC, nD=nD, nRow=nRow, nCol=nCol,
+                    nABCD=title)
+  } else{
+    dpa_dtl <- NULL
   }
-
   # Save the output class
   # ---------------------
   out <- structure(ts,
-                   nA=nA,
-                   nABCD=nABCD,
-                   nLabels=nLabels,
+                   title=title,
+                   analysis=analysis,
                    exposure=exposure,
-                   dpa=dpa)
+                   dpa=dpa,
+                   dpa_detail=dpa_dtl)
   class(out) <- append("mds_ts", class(out))
 
   return(out)
